@@ -96,7 +96,7 @@ struct ring_buffer_per_cpu {
 	ulong real_head_page;
 
 	int head_page_index;
-	unsigned int nr_pages;
+	unsigned long nr_pages;
 	ulong *pages;
 
 	ulong *linear_pages;
@@ -198,7 +198,9 @@ static int init_offsets(void)
 			fprintf(fp, "per cpu buffer sizes\n");
 	}
 
-	if (kernel_symbol_exists("ring_buffer_read"))
+	if (machine_type("PPC64") && kernel_symbol_exists(".ring_buffer_read"))
+		gdb_set_crash_scope(symbol_value(".ring_buffer_read"), ".ring_buffer_read");
+	else if (kernel_symbol_exists("ring_buffer_read"))
 		gdb_set_crash_scope(symbol_value("ring_buffer_read"), "ring_buffer_read");
 
 	if (!per_cpu_buffer_sizes)
@@ -430,7 +432,13 @@ static int ftrace_init_buffers(struct ring_buffer_per_cpu *buffers,
 		buffer_read_value(overrun);
 		buffer_read_value(entries);
 		if (per_cpu_buffer_sizes) {
-			buffer_read_value(nr_pages);
+			if (MEMBER_SIZE("ring_buffer_per_cpu", "nr_pages") == sizeof(unsigned int)) {
+				unsigned int tmp_nr_pages;
+				read_value(tmp_nr_pages, buffers[i].kaddr, ring_buffer_per_cpu, nr_pages);
+				buffers[i].nr_pages = (unsigned long) tmp_nr_pages;
+			} else {
+				buffer_read_value(nr_pages);
+			}
 			pages = buffers[i].nr_pages;
 		} else
 			buffers[i].nr_pages = pages;
@@ -845,8 +853,18 @@ static int syscall_get_enter_fields(ulong call, ulong *fields)
 	inited = 1;
 	data_offset = MAX(MEMBER_OFFSET("ftrace_event_call", "data"), 
 		MEMBER_OFFSET("trace_event_call", "data"));
-	if (data_offset < 0)
-		return -1;
+	if (data_offset < 0) {
+		/*
+		 *  rhel-7.6 moved the .data member into an anonymous union.
+		 */
+		if (MEMBER_EXISTS("ftrace_event_call", "rh_data") &&
+		    MEMBER_EXISTS("ftrace_event_data", "data")) {
+			data_offset = MEMBER_OFFSET("ftrace_event_call", "rh_data") +
+				MEMBER_OFFSET("ftrace_event_data", "data");
+			inited = 2;
+		} else
+			return -1;
+	}
 
 	enter_fields_offset = MEMBER_OFFSET("syscall_metadata", "enter_fields");
 	if (enter_fields_offset < 0)
@@ -860,6 +878,12 @@ work:
 			"read ftrace_event_call data", RETURN_ON_ERROR))
 		return -1;
 
+	if (inited == 2) {
+		if (!readmem(metadata, KVADDR, &metadata, sizeof(metadata),
+		    "read ftrace_event_call data (indirect rh_data)", RETURN_ON_ERROR))
+			return -1;
+	}
+
 	*fields = metadata + enter_fields_offset;
 	return 0;
 }
