@@ -1,8 +1,8 @@
 /* global_data.c - core analysis suite
  *
  * Copyright (C) 1999, 2000, 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010, 2012 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010, 2012 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2006, 2010, 2012-2013, 2018 David Anderson
+ * Copyright (C) 2002-2006, 2010, 2012-2013, 2018 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,23 +72,24 @@ struct command_table_entry linux_command_table[] = {
 	{"*", 	    cmd_pointer, help_pointer, 0},
 	{"alias",   cmd_alias,   help_alias,   0},
         {"ascii",   cmd_ascii,   help_ascii,   0},
+        {"bpf",     cmd_bpf,     help_bpf,     0},
         {"bt",      cmd_bt,      help_bt,      REFRESH_TASK_TABLE},
 	{"btop",    cmd_btop,    help_btop,    0},
 	{"dev",     cmd_dev,     help_dev,     0},
-	{"dis",     cmd_dis,     help_dis,     0},
-	{"eval",    cmd_eval,    help_eval,    0},
-	{"exit",    cmd_quit,    help_exit,    0},
-	{"extend",  cmd_extend,  help_extend,  0},
+	{"dis",     cmd_dis,     help_dis,     MINIMAL},
+	{"eval",    cmd_eval,    help_eval,    MINIMAL},
+	{"exit",    cmd_quit,    help_exit,    MINIMAL},
+	{"extend",  cmd_extend,  help_extend,  MINIMAL},
 	{"files",   cmd_files,   help_files,   REFRESH_TASK_TABLE},
 	{"foreach", cmd_foreach, help_foreach, REFRESH_TASK_TABLE},
 	{"fuser",   cmd_fuser,   help_fuser,   REFRESH_TASK_TABLE},
 	{"gdb",     cmd_gdb,     help_gdb,     REFRESH_TASK_TABLE},
-        {"help",    cmd_help,    help_help,    0},
+        {"help",    cmd_help,    help_help,    MINIMAL},
 	{"ipcs",    cmd_ipcs,    help_ipcs,    REFRESH_TASK_TABLE},
 	{"irq",     cmd_irq,     help_irq,     0},
 	{"kmem",    cmd_kmem,    help_kmem,    0},
 	{"list",    cmd_list,    help__list,   REFRESH_TASK_TABLE},
-	{"log",     cmd_log,     help_log,     0},
+	{"log",     cmd_log,     help_log,     MINIMAL},
 	{"mach",    cmd_mach,    help_mach,    0},
 	{"map",     cmd_map,     help_map,     HIDDEN_COMMAND},
 	{"mod",     cmd_mod,     help_mod,     0},
@@ -99,17 +100,17 @@ struct command_table_entry linux_command_table[] = {
 	{"pte",     cmd_pte,     help_pte,     0},
 	{"ptob",    cmd_ptob,    help_ptob,    0},
 	{"ptov",    cmd_ptov,    help_ptov,    0},
-        {"q",       cmd_quit,    help_quit,    0},
+        {"q",       cmd_quit,    help_quit,    MINIMAL},
         {"tree",    cmd_tree,    help_tree,    REFRESH_TASK_TABLE},
-        {"rd",      cmd_rd,      help_rd,      0},
+        {"rd",      cmd_rd,      help_rd,      MINIMAL},
 	{"repeat",  cmd_repeat,  help_repeat,  0},
 	{"runq",    cmd_runq,    help_runq,    REFRESH_TASK_TABLE},
         {"search",  cmd_search,  help_search,  0},
-        {"set",     cmd_set,     help_set,     REFRESH_TASK_TABLE},
+        {"set",     cmd_set,     help_set,     REFRESH_TASK_TABLE | MINIMAL},
         {"sig",     cmd_sig,     help_sig,     REFRESH_TASK_TABLE},
         {"struct",  cmd_struct,  help_struct,  0},
 	{"swap",    cmd_swap,    help_swap,    0},
-        {"sym",     cmd_sym,     help_sym,     0},
+        {"sym",     cmd_sym,     help_sym,     MINIMAL},
         {"sys",     cmd_sys,     help_sys,     REFRESH_TASK_TABLE},
         {"task",    cmd_task,    help_task,    REFRESH_TASK_TABLE},
 	{"test",    cmd_test,    NULL,         HIDDEN_COMMAND},
