@@ -20,7 +20,11 @@
 #define SPECIFIED_ID      0x1
 #define SPECIFIED_ADDR    0x2
 
-#define IPCS_INIT 0x1
+#define IPCS_INIT  0x1
+#define IDR_ORIG   0x2
+#define IDR_RADIX  0x4
+#define IDR_XARRAY 0x8
+
 #define MAX_ID_SHIFT (sizeof(int)*8 - 1)
 #define MAX_ID_BIT (1U << MAX_ID_SHIFT)
 #define MAX_ID_MASK (MAX_ID_BIT - 1)
@@ -71,19 +75,19 @@ struct ipcs_table {
 	ulong shm_f_op_huge_addr;
 	int use_shm_f_op;
 	int seq_multiplier;
+	int cnt;
+	struct list_pair *lp;
 };
 
 /*
  * function declaration
  */
 
-static void ipcs_init(void);
 static int dump_shared_memory(int, ulong, int, ulong);
 static int dump_semaphore_arrays(int, ulong, int, ulong);
 static int dump_message_queues(int, ulong, int, ulong);
 static int ipc_search_idr(ulong, int, ulong, int (*)(ulong, int, ulong, int, int), int);
 static int ipc_search_array(ulong, int, ulong, int (*)(ulong, int, ulong, int, int), int);
-static ulong idr_find(ulong, int);
 static int dump_shm_info(ulong, int, ulong, int, int);
 static int dump_sem_info(ulong, int, ulong, int, int);
 static int dump_msg_info(ulong, int, ulong, int, int);
@@ -92,13 +96,15 @@ static void get_sem_info(struct sem_info *, ulong, int);
 static void get_msg_info(struct msg_info *, ulong, int);
 static void add_rss_swap(ulong, int, ulong *, ulong *);
 static int is_file_hugepages(ulong);
+static void gather_radix_tree_entries(ulong);
+static void gather_xarray_entries(ulong);
 
 /*
  * global data
  */
 static struct ipcs_table ipcs_table = { 0 };
 
-static void
+void
 ipcs_init(void)
 {
 	if (ipcs_table.init_flags & IPCS_INIT) {
@@ -116,6 +122,7 @@ ipcs_init(void)
 	MEMBER_OFFSET_INIT(idr_layer_layer, "idr_layer", "layer");
 	MEMBER_OFFSET_INIT(idr_layer_ary, "idr_layer", "ary");
 	MEMBER_OFFSET_INIT(idr_top, "idr", "top");
+	MEMBER_OFFSET_INIT(idr_cur, "idr", "cur");
 	MEMBER_OFFSET_INIT(ipc_id_ary_p, "ipc_id_ary", "p");
 	MEMBER_OFFSET_INIT(ipc_ids_entries, "ipc_ids", "entries");
 	MEMBER_OFFSET_INIT(ipc_ids_max_id, "ipc_ids", "max_id");
@@ -185,13 +192,24 @@ ipcs_init(void)
 		ipcs_table.shm_f_op_huge_addr = -1;
 	}
 
-	if (BITS32())
+	if (VALID_MEMBER(idr_layer_ary) && 
+	    get_array_length("idr_layer.ary", NULL, 0) > 64)
+		ipcs_table.idr_bits = 8;
+	else if (BITS32())
 		ipcs_table.idr_bits = 5;
 	else if (BITS64())
 		ipcs_table.idr_bits = 6;
 	else
 		error(FATAL, "machdep->bits is not 32 or 64");
 
+	if (VALID_MEMBER(idr_idr_rt)) {
+		if (STREQ(MEMBER_TYPE_NAME("idr", "idr_rt"), "xarray"))
+			ipcs_table.init_flags |= IDR_XARRAY;
+		else
+			ipcs_table.init_flags |= IDR_RADIX;
+	} else
+		ipcs_table.init_flags |= IDR_ORIG;
+
 	ipcs_table.seq_multiplier = 32768;
 }
 
@@ -574,7 +592,7 @@ ipc_search_array(ulong ipc_ids_p, int specified, ulong specified_value, int (*fn
 static int
 ipc_search_idr(ulong ipc_ids_p, int specified, ulong specified_value, int (*fn)(ulong, int, ulong, int, int), int verbose)
 {
-	int in_use;
+	int i, in_use;
 	ulong ipcs_idr_p;
 	ulong ipc;
 	int next_id, total;
@@ -591,16 +609,40 @@ ipc_search_idr(ulong ipc_ids_p, int specified, ulong specified_value, int (*fn)(
 		return 0;
 	}
 
-	for (total = 0, next_id = 0; total < in_use; next_id++) {
-		ipc = idr_find(ipcs_idr_p, next_id);
-		if (ipc == 0)
-			continue;
-		
-		total++;
-		if (fn(ipc, specified, specified_value, next_id, verbose)) {
-			found = 1;
-			if (specified != SPECIFIED_NOTHING)
-				break;
+	if (VALID_MEMBER(idr_idr_rt)) {
+		switch (ipcs_table.init_flags & (IDR_RADIX|IDR_XARRAY))
+		{
+		case IDR_RADIX: 
+			gather_radix_tree_entries(ipcs_idr_p);
+			break;
+		case IDR_XARRAY:
+			gather_xarray_entries(ipcs_idr_p);
+			break;
+		}
+
+		for (i = 0; i < ipcs_table.cnt; i++) {
+			ipc = (ulong)ipcs_table.lp[i].value;
+			if (fn(ipc, specified, specified_value, UNUSED, verbose)) {
+				found = 1;
+				if (specified != SPECIFIED_NOTHING)
+					break;
+			}
+		}
+
+		if (ipcs_table.lp)
+			FREEBUF(ipcs_table.lp);
+	} else {
+		for (total = 0, next_id = 0; total < in_use; next_id++) {
+			ipc = idr_find(ipcs_idr_p, next_id);
+			if (ipc == 0)
+				continue;
+			
+			total++;
+			if (fn(ipc, specified, specified_value, next_id, verbose)) {
+				found = 1;
+				if (specified != SPECIFIED_NOTHING)
+					break;
+			}
 		}
 	}
 	
@@ -616,7 +658,7 @@ ipc_search_idr(ulong ipc_ids_p, int specified, ulong specified_value, int (*fn)(
 /*
  * search every idr_layer
  */
-static ulong
+ulong
 idr_find(ulong idp, int id)
 {
 	ulong idr_layer_p;
@@ -885,7 +927,7 @@ get_shm_info(struct shm_info *shm_info, ulong shp, int id)
 	add_rss_swap(inodep, is_file_hugepages(filep), &shm_info->rss,
                  &shm_info->swap);
 
-	shm_info->deleted = UINT(buf + OFFSET(shmid_kernel_shm_perm) +
+	shm_info->deleted = UCHAR(buf + OFFSET(shmid_kernel_shm_perm) +
 				OFFSET(kern_ipc_perm_deleted));
 }
 
@@ -930,7 +972,7 @@ get_sem_info(struct sem_info *sem_info, ulong shp, int id)
 
 	sem_info->nsems = ULONG(buf + OFFSET(sem_array_sem_nsems));
 
-	sem_info->deleted = UINT(buf + OFFSET(sem_array_sem_perm) +
+	sem_info->deleted = UCHAR(buf + OFFSET(sem_array_sem_perm) +
 				OFFSET(kern_ipc_perm_deleted));
 }
 
@@ -976,7 +1018,7 @@ get_msg_info(struct msg_info *msg_info, ulong shp, int id)
 
 	msg_info->messages = ULONG(buf + OFFSET(msg_queue_q_qnum));
 	
-	msg_info->deleted = UINT(buf + OFFSET(msg_queue_q_perm) +
+	msg_info->deleted = UCHAR(buf + OFFSET(msg_queue_q_perm) +
 				OFFSET(kern_ipc_perm_deleted));
 }
 
@@ -1083,3 +1125,35 @@ again:
 	return 0;
 }
 
+static void
+gather_radix_tree_entries(ulong ipcs_idr_p)
+{
+	long len;
+
+	ipcs_table.cnt = do_radix_tree(ipcs_idr_p, RADIX_TREE_COUNT, NULL);
+
+	if (ipcs_table.cnt) {
+		len = sizeof(struct list_pair) * (ipcs_table.cnt+1);
+		ipcs_table.lp = (struct list_pair *)GETBUF(len);
+		ipcs_table.lp[0].index = ipcs_table.cnt;
+		ipcs_table.cnt = do_radix_tree(ipcs_idr_p, RADIX_TREE_GATHER, ipcs_table.lp);
+	} else
+		ipcs_table.lp = NULL;
+}
+
+static void
+gather_xarray_entries(ulong ipcs_idr_p)
+{
+	long len;
+
+	ipcs_table.cnt = do_xarray(ipcs_idr_p, XARRAY_COUNT, NULL);
+
+	if (ipcs_table.cnt) {
+		len = sizeof(struct list_pair) * (ipcs_table.cnt+1);
+		ipcs_table.lp = (struct list_pair *)GETBUF(len);
+		ipcs_table.lp[0].index = ipcs_table.cnt;
+		ipcs_table.cnt = do_xarray(ipcs_idr_p, XARRAY_GATHER, ipcs_table.lp);
+	} else
+		ipcs_table.lp = NULL;
+}
+
