@@ -1,7 +1,7 @@
 /* netdump.h
  *
- * Copyright (C) 2002-2009, 2017 David Anderson
- * Copyright (C) 2002-2009, 2017 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002-2009, 2017-2018 David Anderson
+ * Copyright (C) 2002-2009, 2017-2018 Red Hat, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +78,7 @@ struct vmcore_data {
 	ulong backup_src_size;
 	ulonglong backup_offset;
 	ulong arch_data;
+	ulong phys_base;
 };
 
 #define DUMP_ELF_INCOMPLETE  0x1   /* dumpfile is incomplete */
@@ -137,31 +138,11 @@ struct proc_kcore_data {
 	uint flags;
 	uint segments;
 	char *elf_header;
-        Elf64_Ehdr *elf64;
+	size_t header_size;
 	Elf64_Phdr *load64;
-        Elf32_Ehdr *elf32;
+	Elf64_Phdr *notes64;
 	Elf32_Phdr *load32;
+	Elf32_Phdr *notes32;
+	void *vmcoreinfo;
+	uint size_vmcoreinfo;
 };
-
-struct QEMUCPUSegment {
-    uint32_t selector;
-    uint32_t limit;
-    uint32_t flags;
-    uint32_t pad;
-    uint64_t base;
-};
-
-typedef struct QEMUCPUSegment QEMUCPUSegment;
-
-struct QEMUCPUState {
-    uint32_t version;
-    uint32_t size;
-    uint64_t rax, rbx, rcx, rdx, rsi, rdi, rsp, rbp;
-    uint64_t r8, r9, r10, r11, r12, r13, r14, r15;
-    uint64_t rip, rflags;
-    QEMUCPUSegment cs, ds, es, fs, gs, ss;
-    QEMUCPUSegment ldt, tr, gdt, idt;
-    uint64_t cr[5];
-};
-
-typedef struct QEMUCPUState QEMUCPUState;
